Package: rgl
Version: 1.3.36
Title: 3D Visualization Using OpenGL
Authors@R: c(person("Duncan", "Murdoch", role = c("aut", "cre"),
                     email = "murdoch.duncan@gmail.com"),
              person("Daniel", "Adler", role = "aut",
                     email = "dadler@dyncall.org"),
              person("Oleg", "Nenadic", role = "ctb"),
              person("Simon", "Urbanek", role = "ctb"),
              person("Ming", "Chen", role = "ctb"),
              person("Albrecht", "Gebhardt", role = "ctb"),
              person("Ben", "Bolker", role = "ctb"),
              person("Gabor", "Csardi", role = "ctb"),
              person("Adam", "Strzelecki", role = "ctb"),
              person("Alexander", "Senger", role = "ctb"),
              person("The R Core Team", role = c("ctb", "cph")),
              person("Dirk","Eddelbuettel", role = "ctb"),
              person("The authors of Shiny", role = "cph"),
              person("The authors of knitr", role = "cph"),
              person("Jeroen", "Ooms", role = "ctb"),
              person("Yohann", "Demont", role = "ctb"),
              person("Joshua", "Ulrich", role = "ctb"),
              person("Xavier", "Fernandez i Marin", role = "ctb"),
              person("George", "Helffrich", role = "ctb"),
              person("Ivan", "Krylov", role = "ctb"),
              person("Michael", "Sumner", role = "ctb"),
              person("Mike", "Stein", role = "ctb"),
              person("Jonathon", "Love", role = "ctb"),
              person("Mapbox team", role = c("ctb", "cph")))
Depends: R (>= 3.6.0)
Suggests: MASS, rmarkdown (>= 2.16), deldir (>= 1.0-4), orientlib,
        lattice, misc3d, magick, plotrix (>= 3.7-3), interp,
        alphashape3d, tcltk, js (>= 1.2), webshot2 (>= 0.1.0), downlit
        (>= 0.4.0), pkgdown (>= 2.0.0), extrafont, shiny,
        manipulateWidget (>= 0.9.0), testthat, crosstalk, V8, chromote,
        jpeg, png, markdown, xdvir, grid
Imports: graphics, grDevices, stats, utils, htmlwidgets (>= 1.6.0),
        htmltools, knitr (>= 1.33), jsonlite (>= 0.9.20), magrittr, R6,
        base64enc, mime
Enhances: waldo
Description: Provides medium to high level functions for 3D interactive graphics, including
    functions modelled on base graphics (plot3d(), etc.) as well as functions for
    constructing representations of geometric objects (cube3d(), etc.).  Output
    may be on screen using OpenGL, or to various standard 3D file formats including
    WebGL, PLY, OBJ, STL as well as 2D image formats, including PNG, Postscript, SVG, PGF.
License: GPL
URL: https://github.com/dmurdoch/rgl, https://dmurdoch.github.io/rgl/
SystemRequirements: OpenGL and GLU Library (Required for display in R.
        See "Installing OpenGL support" in README.md. Not needed if
        only browser displays using rglwidget() are wanted.), zlib
        (optional), libpng (>=1.2.9, optional), FreeType (optional),
        pandoc (>=1.14, needed for vignettes), LaTeX needed for
        optional latex3d() examples.
BugReports: https://github.com/dmurdoch/rgl/issues
VignetteBuilder: knitr, rmarkdown
Biarch: true
RoxygenNote: 7.3.3
NeedsCompilation: yes
Packaged: 2026-03-05 16:27:45 UTC; murdoch
Author: Duncan Murdoch [aut, cre],
  Daniel Adler [aut],
  Oleg Nenadic [ctb],
  Simon Urbanek [ctb],
  Ming Chen [ctb],
  Albrecht Gebhardt [ctb],
  Ben Bolker [ctb],
  Gabor Csardi [ctb],
  Adam Strzelecki [ctb],
  Alexander Senger [ctb],
  The R Core Team [ctb, cph],
  Dirk Eddelbuettel [ctb],
  The authors of Shiny [cph],
  The authors of knitr [cph],
  Jeroen Ooms [ctb],
  Yohann Demont [ctb],
  Joshua Ulrich [ctb],
  Xavier Fernandez i Marin [ctb],
  George Helffrich [ctb],
  Ivan Krylov [ctb],
  Michael Sumner [ctb],
  Mike Stein [ctb],
  Jonathon Love [ctb],
  Mapbox team [ctb, cph]
Maintainer: Duncan Murdoch <murdoch.duncan@gmail.com>
Repository: CRAN
Date/Publication: 2026-03-06 05:00:02 UTC
Built: R 4.6.0; x86_64-apple-darwin20; 2026-04-28 22:40:55 UTC; unix
Archs: rgl.so.dSYM
