export(.check3d, 
  abclines3d, addNormals, addToSubscene3d, arc3d,
  arrow3d, as.mesh3d, as.rglscene, as.tmesh3d, as.triangles3d, asRow,
  asEuclidean, asEuclidean2, asHomogeneous, asHomogeneous2,
  aspect3d, axes3d, axis3d, box3d,
  bbox3d, bg3d, bgplot3d, 
  Buffer,
  checkDeldir,

  clear3d, clearSubsceneList, clipplanes3d, 
  clipMesh3d, clipObj3d, close3d, compare_proxy.mesh3d,
  contourLines3d,
  cube3d, cuboctahedron3d, cur3d, 
  currentSubscene3d, cylinder3d,
  decorate3d, deform.mesh3d, delFromSubscene3d, divide.mesh3d, dodecahedron3d, dot3d, drape3d, ellipse3d, 
  expect_known_scene, extrude3d,
  facing3d, figWidth, figHeight, filledContour3d,
  gc3d, getBoundary3d,
  getr3dDefaults, getShaders, getWidgetId, gltfTypes,
  GramSchmidt, grid3d, 
  highlevel, hook_rgl, hook_webgl, hover3d,
  icosahedron3d, identify3d, identityMatrix, ids3d,
  in_pkgdown, in_pkgdown_example, latex3d,
  layout3d, legend3d, light3d, lines3d, lowlevel, makeDependency, material3d, 
  mergeVertices, mesh3d, mfrow3d, movie3d, mtext3d, 
  newSubscene3d, next3d, normalize.mesh3d, observer3d, octahedron3d, oh3d, open3d, 
  par3d, par3dinterp, par3dinterpControl,  
  particles3d, pch3d, persp3d, planes3d, 
  play3d, plot3d, plotmath3d, points3d, 
  polygon3d, pop3d, projectDown, 
  qmesh3d, quads3d, readOBJ, readSTL, rgl.abclines, rgl.bbox, rgl.bg, rgl.bringtotop, rgl.clear,
  rgl.getAxisCallback, rgl.getMouseCallbacks, rgl.getWheelCallback, 
  rgl.close, rgl.cur, rgl.ids, rgl.init, rgl.light, rgl.lines, rgl.linestrips,
  rgl.clipplanes, rgl.material, 
  rgl.material.names, rgl.material.readonly,
  rgl.par3d.names, rgl.par3d.readonly,
  rgl.open, rgl.pixels, rgl.planes, rgl.points, rgl.pop, rgl.postscript,
  rgl.primitive, rgl.projection, rgl.quads, rgl.quit, rgl.Sweave, rgl.Sweave.off,
  rgl.select, rgl.select3d, rgl.set, rgl.snapshot, rgl.spheres, rgl.sprites,
  rgl.surface, rgl.texts, rgl.triangles, rgl.user2window,
  rgl.attrib, rgl.attrib.count, rgl.attrib.info, 
  rgl.dev.list, rgl.incrementID, rgl.useNULL,
  rgl.viewpoint, rgl.window2user, rglExtrafonts,
  rglFonts, rglId, rglMouse, rglShared, rglToLattice, rglToBase,
  r3dDefaults, rotate3d, rotationMatrix,
  scale3d, scaleMatrix, scene3d, segments3d, 
  select3d, selectionFunction3d, selectpoints3d,
  rgl.setAxisCallback, rgl.setMouseCallbacks, rgl.setWheelCallback,
  safe.dev.off,
  set3d, setAxisCallbacks, setGraphicsDelay, setupKnitr, 
  setUserCallbacks, setUserShaders, shade3d, shadow3d,

  shapelist3d, shinyGetPar3d, shinySetPar3d, shinyResetBrush, 
  show2d, snapshot3d, 
  spheres3d, spin3d, sprites3d, subdivision3d, 
  subsceneInfo, subsceneList, Sweave.snapshot,
  surface3d, tagged3d, terrain3d, 
  tetrahedron3d, text3d, texts3d, textureSource,
  thigmophobe3d, title3d, 
  tkpar3dsave, tkspinControl, tkspin3d, 
  toggleWidget, triangulate, 
  tmesh3d, transform3d, translate3d, translationMatrix, triangles3d,
  turn3d, useSubscene3d, view3d, wire3d, 
  writeASY, writeOBJ, writePLY, writeSTL, writeWebGL)

 S3method(dot3d, shapelist3d)
 S3method(wire3d, shapelist3d)
 S3method(shade3d, shapelist3d)  
 S3method(translate3d, shapelist3d)
 S3method(rotate3d, shapelist3d)
 S3method(scale3d, shapelist3d)
 S3method(addNormals, shapelist3d)
 
 S3method(dot3d, mesh3d)
 S3method(translate3d, mesh3d)
 S3method(rotate3d, mesh3d)
 S3method(scale3d, mesh3d)
 S3method(merge, mesh3d)
 
 S3method(wire3d, mesh3d)
 S3method(shade3d, mesh3d) 
 S3method(subdivision3d, mesh3d)
 S3method(addNormals, mesh3d)
 
 S3method(plot3d, mesh3d)
 
 S3method(all.equal, mesh3d)
 S3method(all.equal, rglscene)
 
 S3method(as.mesh3d, deldir)
 S3method(as.mesh3d, triSht)
 S3method(as.mesh3d, ashape3d)
 S3method(as.mesh3d, rglId)
 S3method(as.mesh3d, rglobject)
 S3method(as.mesh3d, rglsubscene)
 S3method(as.mesh3d, rglscene)
 S3method(as.mesh3d, mesh3d)
 
 S3method(as.mesh3d, default)
 
 S3method(as.tmesh3d, mesh3d)
 S3method(as.tmesh3d, default)
 
 S3method(as.triangles3d, mesh3d)
 S3method(as.triangles3d, rglId)
 
 S3method(translate3d, default)
 S3method(rotate3d, default)
 S3method(scale3d, default)
 
 S3method(ellipse3d, default)
 S3method(ellipse3d, lm)
 S3method(ellipse3d, glm)
 S3method(ellipse3d, nls)
 
 S3method(plot3d, default)
 S3method(persp3d, default)
 S3method(persp3d, "function")
 S3method(persp3d, deldir)
 S3method(persp3d, triSht)
 S3method(persp3d, ashape3d)
 S3method(persp3d, formula)
 
 S3method(plot3d, rglscene)
 S3method(plot3d, rglobject)
 S3method(plot3d, rglbboxdeco)
 S3method(plot3d, rglbackground)
 S3method(plot3d, rglsubscene)
 S3method(plot3d, rglWebGL)
 S3method(plot3d, "function")
 S3method(plot3d, deldir)
 S3method(plot3d, triSht)
 S3method(plot3d, ashape3d)
 S3method(plot3d, formula)
 S3method(plot3d, lm)
 
 S3method(print, rglscene)
 S3method(print, rglobject)
 S3method(print, rglsubscene)
 S3method(print, rglId)
 S3method(print, rglOpen3d)
 S3method(print, mesh3d)
 S3method(print, shapelist3d)
 S3method(print, rglMouseSelection)
 S3method(print, rglshaders)
 
 S3method(knit_print, rglId)
 S3method(knit_print, rglOpen3d)
 
 S3method(summary, rglscene)
 S3method(summary, rglsubscene)
 
 S3method(drape3d, default)
 S3method(drape3d, mesh3d)

 S3method(contourLines3d, rglId)
 S3method(contourLines3d, mesh3d)
 S3method(filledContour3d, rglId)
 S3method(filledContour3d, mesh3d)
 
 S3method(sew, rglRecordedplot)
 S3method(is_low_change, rglRecordedplot)
 
 if(.Platform$OS.type == "windows") {
   importFrom(utils, getWindowsHandle)
 }
 
 importFrom(graphics, legend, par, plot, plot.new, polygon,
       strwidth, strheight)
 importFrom(grDevices, as.raster, col2rgb, colorRamp, dev.cur, dev.list, dev.new,
       dev.off, dev.prev, dev.set, png, postscript, rgb, xy.coords, xyz.coords)
 importFrom(stats, approxfun, get_all_vars, model.frame, 
      qchisq, qf, splinefun, terms, var)
 importFrom(tools, file_ext)
 importFrom(utils, capture.output, count.fields, file_test, 
            flush.console, packageVersion, read.table, head, tail)
 importFrom(R6, R6Class)
 
# These were in rglwidget
 
export(rglwidget, renderRglwidget, rglwidgetOutput,
       playwidget, renderPlaywidget, playwidgetOutput,
       subsetControl, propertyControl, clipplaneControl, ageControl, vertexControl,
       elementId2Prefix, registerSceneChange, sceneChange,
       "%>%")

importFrom(htmlwidgets, createWidget, prependContent, saveWidget, 
           shinyRenderWidget, shinyWidgetOutput, sizingPolicy)
importFrom(htmltools, css, HTML, htmlDependency, htmlPreserve, img,
           includeScript, tags, tagAppendAttributes,
     tagHasAttribute, tagList, browsable, resolveDependencies)
importFrom(jsonlite, toJSON, base64_dec)
importFrom(knitr, asis_output, fig_path, hook_plot_custom, include_graphics, 
           is_low_change, knit_hooks, 
           knit_meta_add, knit_print, opts_current, opts_hooks, opts_knit, 
           pandoc_to, sew)
importFrom(magrittr, "%>%")
importFrom(stats, coef, predict, residuals)
importFrom(base64enc, base64encode)
